searchData={"items":[{"type":"module","title":"BubbleMatch","doc":"","ref":"BubbleMatch.html"},{"type":"module","title":"Bubblescript Matching Language (BML) - BubbleMatch","doc":"[![Build Status](https://github.com/botsquad/bubble-match/workflows/test/badge.svg)](https://github.com/botsquad/bubble-match)\n[![Hex pm](http://img.shields.io/hexpm/v/bubble_match.svg?style=flat)](https://hex.pm/packages/bubble_match)\n\nBML is a rule language for matching natural language against a rule\nbase. Think of it as regular expressions for _sentences_. Whereas\nregular expressions work on individual characters, BML rules primarily\nwork on a tokenized representation of the string.\n\nBML ships with a builtin string tokenizer, but for production usage\nyou should look into using a language-specific tokenizer, e.g. to use\nthe output of [Spacy's Doc.to_json][spacy] function.\n\n[spacy]: https://spacy.io/api/doc#to_json\n\nThe full documentation on the BML syntax and the API reference is\navailable [on hexdocs.pm](https://hexdocs.pm/bubble_match/). To try\nout BML, [check out the demo\nenvironment](https://bml.botsquad.com/), powered by Phoenix\nLiveview.","ref":"BubbleMatch.html#module-bubblescript-matching-language-bml"},{"type":"module","title":"Examples - BubbleMatch","doc":"Matching basic sequences of words:\n\n| Match string  | Example           | Matches? |\n| ------------- | ----------------- | -------- |\n| `hello world` | Hello, world!     | **yes**  |\n| `hello world` | Well hello world  | **yes**  |\n| `hello world` | hello there world | no       |\n| `hello world` | world hello       | no       |\n\nMatching regular expressions:\n\n| Match string | Example | Matches? |\n| ------------ | ------- | -------- |\n| `/[a-z]+/`   | abcd    | **yes**  |\n\nMatch entities, with the help of Spacy and Duckling preprocessing and\ntokenizing the input:\n\n| Match string | Matches                         | Does not match  |\n| ------------ | ------------------------------- | --------------- |\n| `[person]`   | George Baker                    | Hello world     |\n| `[time]`     | I walked to the store yesterday | My name is John |","ref":"BubbleMatch.html#module-examples"},{"type":"module","title":"Rules overview - BubbleMatch","doc":"The match syntax is composed of adjacent and optionally nested,\nrules. Each individual has the following syntax:\n\n#","ref":"BubbleMatch.html#module-rules-overview"},{"type":"module","title":"Basic words - BubbleMatch","doc":"`hello world`\n\nBasic words; rules consisting of only alphanumeric characters.\n\nMatching is done on both the lowercased, normalized, accents-removed\nversion of the word, and on the lemmatization of the word. The _lemma_\nof a word is its base version; e.g. for verbs it is the root form (are\n→ be, went → go); for nouns it is the singular form of the word.\n\nSome languages (german, dutch, …) have _compound nouns_, that are often\nwritten both with and without spaces or dashes. Use a dash (`-`) to\nmatch on such compound nouns: the rule `was-machine` matches all of\n`wasmachine`, `was-machine` and `was machine`.\n\nThe apostrophe sign is also supported as part of a word, for instance\nwhen matching something like `Martha's cookies`. In this case, the\napostrophe `'s` part is called the _particle_. For places where the\napostrophe is a verb, e.g. in `he'll do that`, you can write the verb\n(\"will\") in full in the BML, as Spacy will determine the proper\nverb. In that case, the BML query would be `he will do that`, which\nwould also match the version with the apostrophe. Same goes for\n`don't`, `he's`, etc.\n\n#","ref":"BubbleMatch.html#module-basic-words"},{"type":"module","title":"Literals - BubbleMatch","doc":"`\"Literal word sequence\"`\n\nMatches a literal piece of text, which can span multiple\ntokens. Matching is **case insensitive**, and also insensitive to\nthe presence of accented characters.\n\n#","ref":"BubbleMatch.html#module-literals"},{"type":"module","title":"Ignoring tokens: \\_ - BubbleMatch","doc":"`hello _ world`\n\nThe standalone occurence of `_` matches 0-5 of any available token,\nnon-greedy. This can be used in places where you expect a few tokens\nto occur but you don't care about the tokens.\n\n#","ref":"BubbleMatch.html#module-ignoring-tokens-_"},{"type":"module","title":"Matching a range of tokens - BubbleMatch","doc":"- `[1]` match exactly one token; any token\n- `[2+]` match 2 or more tokens (greedy)\n- `[1-3]` match 1 to 3 tokens (greedy)\n- `[2+?]` match 2 or more tokens (non-greedy)\n- `[1-3?]` match 1 to 3 tokens (non-greedy)\n\nUse this when you know how many tokens you need to match, but it does\nnot matter what the contents of the tokens is.\n\n#","ref":"BubbleMatch.html#module-matching-a-range-of-tokens"},{"type":"module","title":"Entities - BubbleMatch","doc":"Entity tokens: `[email]` matches a token of type `:entity` with\nvalue.kind == `email`. Entities are extracted by external means,\ne.g. by an NLP NER engine like Duckling.\n\nEntities are automatically captured under a variable with the same\nname as the entity's kind.\n\nThe default list of supported entities is the following:\n\n- `amount_of_money` (duckling)\n- `credit_card_number` (duckling)\n- `date` (spacy)\n- `distance` (duckling)\n- `duration` (duckling)\n- `email` (duckling)\n- `event` (spacy)\n- `fac` (spacy)\n- `gpe` (spacy)\n- `language` (spacy)\n- `law` (spacy)\n- `loc` (spacy)\n- `money` (spacy)\n- `norp` (spacy)\n- `number` (duckling)\n- `ordinal` (duckling)\n- `org` (spacy)\n- `percent` (spacy)\n- `person` (spacy)\n- `phone_number` (duckling)\n- `product` (spacy)\n- `quantity` (duckling)\n- `temperature` (duckling)\n- `time` (duckling)\n- `url` (duckling)\n- `volume` (duckling)\n- `work_of_art` (spacy)\n\nFrom our experience, Duckling entities work much better than Spacy\nentities, and are preferred for use. Besides being more accurate, the\nDuckling entities also provide more metadata, like valid UTC times\nwhen a date is recognized.\n\n#","ref":"BubbleMatch.html#module-entities"},{"type":"module","title":"Regular expressions - BubbleMatch","doc":"`/regex/`\n\nMatches the given regex against the sentence. Regexes can span\nmultiple tokens, thus you can match on whitespace and other token\nseparators. Regular expressions are **case insensitive**.\n\nRegular expression named capture groups are also supported, to capture\na specific part of a string: `/KL(? \\d+)/` matches\nKL12345 and extracts `12345` as the `flight_number` capture.\n\n#","ref":"BubbleMatch.html#module-regular-expressions"},{"type":"module","title":"Per-token regular expressions - BubbleMatch","doc":"`/regex/T`\n\nThe special regex flag `T` is used to indicate that the regex should be run\nagainst a single token instead of against the raw text of the sentence.\n\nThis will make the regex capturing much more 'narrow'. The regex start and end\nsymbols (`^` and `$`) are automatically added to the regex, so eg the BML\n`/\\d{4}/T` will match the token \"1234\" but _not_ \"12345\".\n\n#","ref":"BubbleMatch.html#module-per-token-regular-expressions"},{"type":"module","title":"OR / grouping construct - BubbleMatch","doc":"Use parentheses combined with the pipe `|` character to specify an OR clause.\n\n- `pizza | fries | chicken` - OR-clause on the root level without\n  parens, matches either token\n\n- `a ( a | b | c )` - use parentheses to separate OR-clauses;\n  matches one token consisting of first `a`, and then `a`, `b` or\n  `c`.\n\n- `( hi | hello )[=greeting]` matches 1 token and stores it in `greeting`\n\nParenthesis with | can also be used to capture a sequence of tokens together as one group:\n\n- `( a )[3+]` matches 3 or more token consisting of `a`\n\n#","ref":"BubbleMatch.html#module-or-grouping-construct"},{"type":"module","title":"Permutation construct - BubbleMatch","doc":"The permutation construct using pointy brackets, ` ` matches the\ngiven rules in no particular order.\n\n` ` matches any permutation of the sequence `a b c`; `a c b`, or `b a c`, or `c a b`, etc\n\nAn implicit `_` is inserted between all rules. So the rule ` ` can\nalso be written as `(a _ b | b _ a)`.\n\n#","ref":"BubbleMatch.html#module-permutation-construct"},{"type":"module","title":"Start / end sentence markers - BubbleMatch","doc":"To match the beginning of end of sentences, the following constructs can be used:\n\n- `[Start]` Matches the start of a sentence\n- `[End]` Matches the end of a sentence\n\n> The `[Start]` and `[End]` symbols are not always the same as the\n> start and end of the input string, as sometimes the user input is\n> split into multiple sentences, based on the Spacy sentence\n> tokenizer.\n\n#","ref":"BubbleMatch.html#module-start-end-sentence-markers"},{"type":"module","title":"Part-of-speech tags (word kinds) - BubbleMatch","doc":"To be able to disambiguate between word kinds, the `%` construct\nmatches on the POS-tag of a token:\n\n- `%VERB` matches any verb\n- `%NOUN` matches any noun\n\nAny other [POS Spacy tags](https://spacy.io/api/annotation#pos-en) are\nvalid as well.\n\n#","ref":"BubbleMatch.html#module-part-of-speech-tags-word-kinds"},{"type":"module","title":"Optionality modifier - BubbleMatch","doc":"An appended `?` makes the given rule optional (it needs to occur 0 or 1 times).\n\n#","ref":"BubbleMatch.html#module-optionality-modifier"},{"type":"module","title":"Repetition modifier - BubbleMatch","doc":"Any rule can have a `[]` block which contains a repetition modifier\nand/or a capture expression.\n\n- `a[1]` match exactly one `a` word\n- `a[2+]` match 2 or more `a`'s (greedy)\n- `a[1-3]` match 1 to 3 `a`'s (greedy)\n- `a[2+?]` match 2 or more `a`'s (non-greedy)\n- `a[1-3?]` match 1 to 3 `a`'s (non-greedy)\n\n#","ref":"BubbleMatch.html#module-repetition-modifier"},{"type":"module","title":"Capture modifier - BubbleMatch","doc":"`(my name is _)[=x]` stores the entire token sequence \"My name is john\"\n\n#","ref":"BubbleMatch.html#module-capture-modifier"},{"type":"module","title":"Punctuation - BubbleMatch","doc":"Punctuation is optional, and can be ignored while creating match\nrules. However, punctuation tokens _are_ stored in the tokenized\nversion of the input; in fact, multiple _tokenizations_ of the input\nare stored for each sentence, one without and one with with the\npunctuation.\n\nThe sentence `Hello, world.` is stored both as:\n\n- `Hello` `world`\n- `Hello` `,` `world` `.`\n\nMatching punctuation can be done by including the punctuation into `'`\nliteral quotes.","ref":"BubbleMatch.html#module-punctuation"},{"type":"module","title":"Sentence tokenization - BubbleMatch","doc":"The expression matching works on a per-sentence basis; the idea is\nthat it does not make sense to create expressions that span over\nsentences.\n\nThe builtin sentence tokenizer (`BubbleMatch.Sentence.Tokenizer`) does\n**not** have the concept of sentences, and thus treats each input as a\nsingle sentence, even in the existence of periods in the input.\n\nHowever, the prefered way of using this library is by running the\ninput through an NLP preprocessor like Spacy, which does tokenize an\ninput into individual sentences.","ref":"BubbleMatch.html#module-sentence-tokenization"},{"type":"module","title":"Sigil - BubbleMatch","doc":"For use within Elixir, it is possible to use a `~m` sigil which parses\nthe given BML query on compile-time:\n\n```elixir\ndefmodule MyModule do\n  use BubbleMatch.Sigil\n\n  def greeting?(input) do\n    BubbleMatch.match(~m\"hello | hi | howdy\", input) != :nomatch\n  end\nend\n```","ref":"BubbleMatch.html#module-sigil"},{"type":"module","title":"Installation - BubbleMatch","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `bubble_match` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bubble_match, \"~> 0.1.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at [https://hexdocs.pm/bubble_match](https://hexdocs.pm/bubble_match).","ref":"BubbleMatch.html#module-installation"},{"type":"function","title":"BubbleMatch.match/2","doc":"Match a given input against a BML query.","ref":"BubbleMatch.html#match/2"},{"type":"function","title":"BubbleMatch.parse/2","doc":"Parse a string into a BML expression.","ref":"BubbleMatch.html#parse/2"},{"type":"function","title":"BubbleMatch.parse!/2","doc":"Parse a string into a BML expression, raises on error.","ref":"BubbleMatch.html#parse!/2"},{"type":"type","title":"BubbleMatch.input/0","doc":"","ref":"BubbleMatch.html#t:input/0"},{"type":"type","title":"BubbleMatch.match_result/0","doc":"","ref":"BubbleMatch.html#t:match_result/0"},{"type":"type","title":"BubbleMatch.parse_opt/0","doc":"","ref":"BubbleMatch.html#t:parse_opt/0"},{"type":"type","title":"BubbleMatch.parse_opts/0","doc":"","ref":"BubbleMatch.html#t:parse_opts/0"},{"type":"type","title":"BubbleMatch.t/0","doc":"","ref":"BubbleMatch.html#t:t/0"},{"type":"module","title":"BubbleMatch.Entity","doc":"","ref":"BubbleMatch.Entity.html"},{"type":"function","title":"BubbleMatch.Entity.new/5","doc":"","ref":"BubbleMatch.Entity.html#new/5"},{"type":"module","title":"BubbleMatch.Sentence","doc":"A data structure which holds a tokenized sentence.\n\nThe struct contains the text of the sentence (in the *text*\nproperty), and a list of *tokenizations*. Normally, a sentence has\njust one tokenization, but adding entities to the sentence might\ncause several tokens in the sentence to be replaed with an entity\ntoken, thus creating the need for multiple tokenizations (as you\nstill might want to match on the original sentence, e.g. in the case\nof a falsely identified entitiy)","ref":"BubbleMatch.Sentence.html"},{"type":"function","title":"BubbleMatch.Sentence.add_duckling_entities/2","doc":"Enrich the given sentence with entities extracted via Duckling\n\nThis function takes the output of the [Duckling JSON\nformat][duckling] and enriches the given sentence with the entities\nthat were found using Duckling.\n\n[duckling]: https://github.com/facebook/duckling","ref":"BubbleMatch.Sentence.html#add_duckling_entities/2"},{"type":"function","title":"BubbleMatch.Sentence.from_spacy/1","doc":"Convert a JSON blob from Spacy NLP data into a sentence.\n\nThis function takes the output of Spacy's [Doc.to_json][spacy]\nfunction and converts it into a sentence.\n\nNote that the Spacy tokenizer detects multiple sentences. However,\nin many cases the result is suboptimal and therefore we always\nconstruct a single sentence, given our use case of chat messages.\n\n[spacy]: https://spacy.io/api/doc#to_json","ref":"BubbleMatch.Sentence.html#from_spacy/1"},{"type":"function","title":"BubbleMatch.Sentence.naive_tokenize/1","doc":"Tokenize an input into individual tokens.\n\nAs the name suggests, this tokenization is quite naive. It only\nsplits strings on whitespace and punctuation, disregarding any\nlanguage-specific information. However, for 'basic' use cases, and\nfor our test suite, it is good enough.","ref":"BubbleMatch.Sentence.html#naive_tokenize/1"},{"type":"type","title":"BubbleMatch.Sentence.t/0","doc":"","ref":"BubbleMatch.Sentence.html#t:t/0"},{"type":"module","title":"BubbleMatch.Sigil","doc":"","ref":"BubbleMatch.Sigil.html"},{"type":"macro","title":"BubbleMatch.Sigil.sigil_m/2","doc":"Define the `~m` sigil for compile-time parsing of BML expressions.\n\nFor use within Elixir it is possible to use a `~m` sigil which\nparses the given BML query on compile-time:\n\n```elixir\ndefmodule MyModule do\n  use BubbleMatch.Sigil\n\n  def greeting?(input) do\n    BubbleMatch.match(~m\"hello | hi | howdy\", input) != :nomatch\n  end\nend\n```","ref":"BubbleMatch.Sigil.html#sigil_m/2"},{"type":"module","title":"BubbleMatch.Token","doc":"A token is a single word or a part of the sentence. A sentence is a sequence of tokens.\n\nEach token contains information and metadata that is used to match\nsentences on, and to extract information from.","ref":"BubbleMatch.Token.html"},{"type":"function","title":"BubbleMatch.Token.base_form/1","doc":"Get the base form of the given string; the downcased, ASCII version.","ref":"BubbleMatch.Token.html#base_form/1"},{"type":"function","title":"BubbleMatch.Token.entity?/2","doc":"Test whether a token is an entity of the given kind.","ref":"BubbleMatch.Token.html#entity?/2"},{"type":"function","title":"BubbleMatch.Token.from_duckling_entity/1","doc":"Constructs a token from a Duckling entity definition","ref":"BubbleMatch.Token.html#from_duckling_entity/1"},{"type":"function","title":"BubbleMatch.Token.from_spacy/1","doc":"Given a single token in Spacy's JSON format, convert it into a token.","ref":"BubbleMatch.Token.html#from_spacy/1"},{"type":"function","title":"BubbleMatch.Token.from_spacy_entity/2","doc":"Constructs a token from a Spacy entity definition","ref":"BubbleMatch.Token.html#from_spacy_entity/2"},{"type":"function","title":"BubbleMatch.Token.pos?/2","doc":"Test whether a token mathces the given POS (part-of-speech) tag.","ref":"BubbleMatch.Token.html#pos?/2"},{"type":"function","title":"BubbleMatch.Token.punct?/1","doc":"Test whether a token is punctuation","ref":"BubbleMatch.Token.html#punct?/1"},{"type":"function","title":"BubbleMatch.Token.word?/2","doc":"Test whether a token matches the given (optionally normalized) word.","ref":"BubbleMatch.Token.html#word?/2"},{"type":"type","title":"BubbleMatch.Token.t/0","doc":"Tokens contain the following fields:\n\n * `raw` - the raw text value of the token, including any surrounding\n   whitespace.\n\n * `value` - the normalized value of the token. In the case of word\n   tokens, this is usually the normalized, lowercased version of the\n   word. In the case of entities, this value holds a map with keys\n   `kind`, `provider` and `value`.\n\n* `start` - the start index; where in the original sentence the\n  token starts.\n\n* `end` - the end index; where in the original sentence the\n  token ends.\n\n* `index` - the (zero-based) token index number; 0 if it's the first\n   token, 1 if it's the second, etc.\n\n* `type` - the type of the token; an atom, holding either `:entity`,\n  `:spacy`, `:naive`, depending on the way the token was\n  originally created.","ref":"BubbleMatch.Token.html#t:t/0"},{"type":"module","title":"BubbleMatch.Unidekode","doc":"Documentation for Unidekode.","ref":"BubbleMatch.Unidekode.html"},{"type":"function","title":"BubbleMatch.Unidekode.drop_accented/1","doc":"Remove accented characters from the string, but keeping special characters like emoji","ref":"BubbleMatch.Unidekode.html#drop_accented/1"},{"type":"function","title":"Examples - BubbleMatch.Unidekode.drop_accented/1","doc":"iex> BubbleMatch.Unidekode.drop_accented(\"código\")\n    \"codigo\"\n\n    iex> BubbleMatch.Unidekode.drop_accented(\"código 👍\")\n    \"codigo 👍\"\n\n    iex> BubbleMatch.Unidekode.drop_accented(\"éé немає 👍\")\n    \"ee немає 👍\"","ref":"BubbleMatch.Unidekode.html#drop_accented/1-examples"},{"type":"function","title":"BubbleMatch.Unidekode.to_ascii/1","doc":"Transliterate Unicode characters to US-ASCII.","ref":"BubbleMatch.Unidekode.html#to_ascii/1"},{"type":"function","title":"Examples - BubbleMatch.Unidekode.to_ascii/1","doc":"iex> BubbleMatch.Unidekode.to_ascii(\"código\")\n    \"codigo\"\n\n    iex> BubbleMatch.Unidekode.to_ascii(\"código😀\")\n    \"codigo\"\n\n    iex> BubbleMatch.Unidekode.to_ascii('código')\n    'codigo'\n\n    iex> BubbleMatch.Unidekode.to_ascii('código℗')\n    'codigo'","ref":"BubbleMatch.Unidekode.html#to_ascii/1-examples"}],"content_type":"text/markdown"}